//****************************************************************************//
//                                                                            //
//                                TXYZPlot                                    //
//               A Borland C++ Builder 3D Plotting Component                  //
//               -------------------------------------------                  //
//               TXYZPlot is copyright 1998 Jeff Nicholson                    //
//                             jeffn@rt66.com                                 //
//                                                                            //
//   This software is provided "as is" without express or implied warranty.   //
//****************************************************************************//
#ifndef XYZPlotH
#define XYZPlotH
//---------------------------------------------------------------------------
#include <vcl\SysUtils.hpp>
#include <vcl\Controls.hpp>
#include <vcl\Classes.hpp>
#include <vcl\Forms.hpp>
//---------------------------------------------------------------------------
enum T3DPlotStyle {psPoints, psLines, psLinesPoints, psGrid, psSurface};
//---------------------------------------------------------------------------
enum T3DAxisStyle {asBox, asXYZ, asXYZPlane, asBoxAndPlane, asNone};
//---------------------------------------------------------------------------
enum TMouseFunction {mfDrag, mfZoom, mfRotate};
//---------------------------------------------------------------------------
class TXYPoint
{
public:
   TXYPoint(void)
   {
      X = 0;
      Y = 0;
   };
   TXYPoint(double xx, double yy)
   {
      X = xx;
      Y = yy;
   };
   double X;
   double Y;
};// end TXYPoint
//---------------------------------------------------------------------------
class TXYZPoint
{
public:
   TXYZPoint(double xx, double yy, double zz)
   {
     X = xx;
     Y = yy;
     Z = zz;
   };
   TXYZPoint(TXYZPoint* Point)
   {
      X = Point->X;
      Y = Point->Y;
      Z = Point->Z;
   };
   TXYZPoint()
   {
      X = 0;
      Y = 0;
      Z = 0;
   };
   void SetPoint(double xx, double yy, double zz)
   {
     X = xx;
     Y = yy;
     Z = zz;
   };
   void RotateX(double Theta);
   void RotateY(double Theta);
   void RotateZ(double Theta);
   double X;
   double Y;
   double Z;
};// end TXYZPoint
//---------------------------------------------------------------------------
// Class definition for a member of a TList of Plots
class T3DPlotListMember
{
public:
   T3DPlotListMember(void)
   {
      z = NULL;
   };
   T3DPlotListMember(TXYZPoint* z_in, int XPoints, int YPoints,
                           int PNumber, T3DPlotStyle PStyle, TColor LColor,
                           TColor SColor);
   __fastcall T3DPlotListMember(const T3DPlotListMember* PlotListMember);
   ~T3DPlotListMember(void){delete[] z;};
   bool AddPlot(TXYZPoint* z_in, int XPoints, int YPoints,
                           int PNumber, T3DPlotStyle PStyle,
                           TColor LColor, TColor SColor, TList* PlotList);
   bool RemovePlot(int PNumber, TList* PlotList);
   TXYZPoint* z;    // array of data
   TColor LineColor;    // color of plot
   TColor FillColor;
   int NumXPoints;   // number of plot points
   int NumYPoints;   // number of plot points
   int NumPoints;
   T3DPlotStyle PlotStyle;
   int PlotIndex;       // index of the plot
};
//------------------------------------------------------------------------
class TXYZPlot : public TPaintBox
{
private:
protected:
   Graphics::TBitmap* PaintPage;
   TList* XYZPlotList;
   virtual void __fastcall SetVanishingPoint(double VP);
   virtual void __fastcall SetPerspectiveOn(bool PO);
   virtual void __fastcall SetMouseFunction(TMouseFunction MF);
   virtual void __fastcall SetXMax(double value);
   virtual void __fastcall SetYMax(double value);
   virtual void __fastcall SetZMax(double value);
   virtual void __fastcall SetXMin(double value);
   virtual void __fastcall SetYMin(double value);
   virtual void __fastcall SetZMin(double value);
   virtual void __fastcall SetXAxisColor(TColor AC);
   virtual void __fastcall SetYAxisColor(TColor AC);
   virtual void __fastcall SetZAxisColor(TColor AC);
   virtual void __fastcall SetBoxColor(TColor BC);
   virtual void __fastcall SetPlaneColor(TColor PC);
   virtual void __fastcall SetShowLabels(bool SL);
   virtual void __fastcall SetShowXLabel(bool XL);
   virtual void __fastcall SetShowYLabel(bool YL);
   virtual void __fastcall SetShowZLabel(bool ZL);
   virtual void __fastcall SetShowOrigin(bool SO);
   virtual void __fastcall SetPointSize(int PS);
   virtual void __fastcall SetPhi(double angle);
   virtual void __fastcall SetTheta(double angle);
   virtual void __fastcall SetPsi(double angle);
   virtual void __fastcall SetFitToScreen(bool FS);
   virtual void __fastcall SetAutoScale(bool AS);
   virtual void __fastcall SetXLabelPrecision(int LP);
   virtual void __fastcall SetYLabelPrecision(int LP);
   virtual void __fastcall SetZLabelPrecision(int LP);
   virtual void __fastcall SetXLabelFormat(AnsiString::TStringFloatFormat LF);
   virtual void __fastcall SetYLabelFormat(AnsiString::TStringFloatFormat LF);
   virtual void __fastcall SetZLabelFormat(AnsiString::TStringFloatFormat LF);
   virtual void __fastcall SetVisible(bool V);
   virtual void __fastcall SetAxisStyle(T3DAxisStyle AS);
   virtual void __fastcall SetAutoRedraw(bool AR);
   void RedrawScreen(void);
   bool FindPlot(T3DPlotListMember* &PlotPtr, int PNumber);
   void DrawGrid(T3DPlotListMember* PlotPtr, int i, int j);
   void DrawPolygon(T3DPlotListMember* PlotPtr, int i, int j);
   void DrawAllPlots(void);
   void DrawOrigin(void);
   void DrawOnePlot(T3DPlotListMember* PlotPtr);
   void ClearPlot(void);
   inline void DrawAxis(void);
   void DrawBackAxis(void);
   void DrawFrontAxis(void);
   inline void DrawXAxis(void);
   inline void DrawYAxis(void);
   inline void DrawZAxis(void);
   inline void DrawXZPlane(void);
   inline void DrawYZPlane(void);
   inline void DrawXYPlane(void);
   void DrawXLabel(void);
   void DrawYLabel(void);
   void DrawZLabel(void);
   inline void DrawAxisPlane(void);
   void CalculateAxisArray(void);
   void CalculateAxisPlane(TPoint Points[5],int FP, int Change[5]);
   inline TXYPoint LocalToScreen(TXYPoint P1);
   void FindXYZMaxMin(void);
   void ScaleScreen(void);
   int ConvertFarPoint(int Center,int Change);
   TXYZPoint LocalToGlobal(TXYZPoint P1);
   TXYPoint ConvertTo2D(TXYZPoint* P1);

   Classes::TNotifyEvent FOnPaint;
   T3DAxisStyle FAxisStyle;
   TXYZPoint AxisArray[8];
   TXYZPoint XYSurface[4];
   int DistantPoint;
   int DistantPoint2;
   int FarPoint;
   TXYZPoint* Origin;     // origin of coordinate system
   TColor FXAxisColor;
   TColor FYAxisColor;
   TColor FZAxisColor;
   TColor FBoxColor;
   TColor FPlaneColor;
   bool FAutoScale;
   bool FFitToScreen;
   double FPhi;            // angular orientation of coordinate system
   double FTheta;
   double FPsi;
   double FXMax;
   double FYMax;
   double FZMax;
   double FXMin;
   double FYMin;
   double FZMin;
   int FPointSize;                       // size of point when PlotStyle = Points
   bool FMouseControl;
   TMouseFunction FMouseFunction;
   double FVanishingPoint;
   bool FPerspectiveOn;
   bool FShowLabels;
   bool FShowXLabel;                     // Set to true to show the x labels
   bool FShowYLabel;                     // Set to true to show the y labels
   bool FShowZLabel;                     // Set to true to show the y labels
   bool FShowOrigin;
   int FXLabelPrecision;
   int FYLabelPrecision;
   int FZLabelPrecision;
   AnsiString::TStringFloatFormat FYLabelFormat;
   AnsiString::TStringFloatFormat FXLabelFormat;
   AnsiString::TStringFloatFormat FZLabelFormat;
   bool FShowAxis;
   bool FAutoRedraw;
   double MaxXValue;
   double MaxYValue;
   double MinXValue;
   double MinYValue;
   int MouseX;
   int MouseY;                   // coordinates of the mouse when dragging and zooming th plot
   bool DragXY;                        // true if mouse button is pressed
   bool DragXZ;                        // true if alt key and mouse button pressed
   bool ZoomXY;
   bool ZoomXZ;
   bool RotatePhiPsi;
   bool RotatePhiTheta;
public:
   __fastcall TXYZPlot(TComponent* Owner);
   __fastcall TXYZPlot(TXYZPlot& XYZPlot);
   __fastcall virtual ~TXYZPlot(void);
   void AssignPlot(const TXYZPlot* XYZPlot);
   void AssignPlotData(const TXYZPlot* XYPZlot);
   void AppendPlotData(const TXYZPlot* XYZPlot);
   void AssignPlotProperties(const TXYZPlot* XYZPlot);
   bool XYZPlot(double z_array[], int NumXPoints, int NumYPoints,
                           int PNumber=0, T3DPlotStyle PStyle = psSurface,
                           TColor LColor = clRed, TColor SColor = clBlue);
   bool XYZPlot(double z_array[], int NumXPoints, int NumYPoints,
                           double MinX, double MaxX, double MinY, double MaxY,
                           int PNumber=0, T3DPlotStyle PStyle = psSurface,
                           TColor LColor = clRed, TColor SColor = clBlue);
   bool XYZPlot(double z_array[][3], int NumPoints,
                           int PNumber=0, T3DPlotStyle PStyle = psLines,
                           TColor LColor = clRed, TColor SColor = clBlue);
   bool RemovePlot(int PNumber=0);
   void SetAngles(double phi, double theta, double psi);
   bool SetPlotLineColor(TColor PColor, int PNumber=0);
   bool SetPlotFillColor(TColor PColor, int PNumber = 0);
   bool SetPlotStyle(T3DPlotStyle PStyle, int PNumber = 0);
   void Redraw(void);


   // new event handlers
   virtual void __fastcall Paint(void);
   virtual void __fastcall MouseMove(Classes::TShiftState Shift, int X, int Y);
   virtual void __fastcall MouseDown(TMouseButton Button,
	TShiftState Shift, int X, int Y);
   virtual void __fastcall MouseUp(TMouseButton Button,
	TShiftState Shift, int X, int Y);
   virtual void __fastcall DblClick(void);
   __property Canvas ;

__published:
   __property TColor XAxisColor = {read = FXAxisColor, write = SetXAxisColor, default = clRed};
   __property TColor YAxisColor = {read = FYAxisColor, write = SetYAxisColor, default = clGreen};
   __property TColor ZAxisColor = {read = FZAxisColor, write = SetZAxisColor, default = clBlue};
   __property TColor BoxColor = {read = FBoxColor, write = SetBoxColor, default = clBlack};
   __property TColor PlaneColor = {read = FPlaneColor, write = SetPlaneColor, default = clYellow};
   __property double XMax = {read = FXMax, write = SetXMax, default = 1};
   __property double YMax = {read = FYMax, write = SetYMax, default = 1};
   __property double ZMax = {read = FZMax, write = SetZMax, default = 1};
   __property double XMin = {read = FXMin, write = SetXMin, default = 0};
   __property double YMin = {read = FYMin, write = SetYMin, default = 0};
   __property double ZMin = {read = FZMin, write = SetZMin, default = 0};
   __property double Phi = {read = FPhi, write = SetPhi};
   __property double Theta = {read = FTheta, write = SetTheta};
   __property double Psi = {read = FPsi, write = SetPsi};
   __property bool AutoScale = {read = FAutoScale, write = SetAutoScale, default = true};
   __property bool FitToScreen = {read = FFitToScreen, write = SetFitToScreen, default = true};
   __property bool MouseControl = {read = FMouseControl, write = FMouseControl, default = true};
   __property TMouseFunction MouseFunction = {read = FMouseFunction, write = FMouseFunction, default = mfRotate};
   __property int PointSize = {read = FPointSize, write = SetPointSize, default = 1};
   __property bool PerspectiveOn = {read = FPerspectiveOn, write = SetPerspectiveOn, default = true};
   __property double VanishingPoint = {read = FVanishingPoint, write = SetVanishingPoint, default = 100};
   __property int XLabelPrecision = {read = FXLabelPrecision, write = SetXLabelPrecision, default = 4};
   __property int YLabelPrecision = {read = FYLabelPrecision, write = SetYLabelPrecision, default = 4};
   __property int ZLabelPrecision = {read = FZLabelPrecision, write = SetZLabelPrecision, default = 4};
   __property AnsiString::TStringFloatFormat XLabelFormat = {read = FXLabelFormat, write = SetXLabelFormat, default = AnsiString::sffGeneral};
   __property AnsiString::TStringFloatFormat YLabelFormat = {read = FYLabelFormat, write = SetYLabelFormat, default = AnsiString::sffGeneral};
   __property AnsiString::TStringFloatFormat ZLabelFormat = {read = FZLabelFormat, write = SetZLabelFormat, default = AnsiString::sffGeneral};
   __property bool ShowXLabel = {read = FShowXLabel, write = SetShowXLabel, default = true};
   __property bool ShowYLabel = {read = FShowYLabel, write = SetShowYLabel, default = true};
   __property bool ShowZLabel = {read = FShowZLabel, write = SetShowZLabel, default = true};
   __property bool ShowLabels = {read = FShowLabels, write = SetShowLabels};
   __property bool ShowOrigin = {read = FShowOrigin, write = SetShowOrigin, default = true};
   __property T3DAxisStyle AxisStyle = {read = FAxisStyle, write = SetAxisStyle, default = asBox};
   __property bool AutoRedraw = {read = FAutoRedraw, write = SetAutoRedraw, default = true};
   __property Height= {default = 150};
   __property Width= {default = 200};
};
//---------------------------------------------------------------------------
#endif
